from .race_class import RaceClass
from .race_grade import RaceGrade


class RaceLevel:
    def __init__(self, value: str | RaceGrade | RaceClass):
        if isinstance(value, RaceClass) and value.value == 1:
            raise ValueError("Class 1 race needs a specified grade")

        self.grade = (
            value
            if isinstance(value, RaceGrade)
            else RaceGrade(value if "G" in str(value) else None)
        )
        self.class_ = (
            value
            if isinstance(value, RaceClass)
            else RaceClass(1)
            if self.grade
            else RaceClass(value)
        )

    def __repr__(self):
        return f"<RaceLevel: {repr(self.grade) if self.grade else repr(self.class_)}>"

    def __str__(self):
        return f"({self.class_.value}) {self.grade if self.grade.value else ''}".strip()

    def __eq__(self, other):
        return self.grade == other.grade and self.class_ == other.class_

    def __gt__(self, other):
        return self.grade > other.grade or self.class_ > other.class_

    def __lt__(self, other):
        return self.grade < other.grade or self.class_ < other.class_



from peak_utility.number import Ordinal  # type: ignore


class FinishingPosition(Ordinal):
    """
    A class that represents the finishing position of a horse in a race.

    """

    def __new__(cls, value):
        if int(value) < 0:
            raise ValueError("Finishing position cannot be negative.")
        return super().__new__(cls, value)

    def __bool__(self):
        return int(self) >= 0

    def __repr__(self):
        if int(self) == 0:
            return "<FinishingPosition: Unplaced>"
        return f"<FinishingPosition: {super().__repr__()}>"

    def __str__(self):
        if int(self) == 0:
            return "Unplaced"
        return super().__repr__()

    def __add__(self, other: int) -> int:
        raise TypeError("FinishingPosition cannot be added to.")

    def __sub__(self, other: int) -> int:
        raise TypeError("FinishingPosition cannot be subtracted from.")

    def __mul__(self, other: int) -> int:
        raise TypeError("FinishingPosition cannot be multiplied.")

    def __truediv__(self, other: int) -> int:
        raise TypeError("FinishingPosition cannot be divided.")

    def __eq__(self, other: object) -> bool:
        return isinstance(other, (int, FinishingPosition)) and int(self) == int(other)

    def __lt__(self, other: int) -> bool:
        return int(self) > int(other)

    def __le__(self, other: int) -> bool:
        return int(self) >= int(other)

    def __gt__(self, other: int) -> bool:
        return int(self) < int(other)

    def __ge__(self, other: int) -> bool:
        return int(self) <= int(other)



import re
from .racing_code import RacingCode


class RaceGrade:
    REGEX = r"G(?:roup|rade|)\s*"

    def __init__(self, grade: str | int | None, racing_code: RacingCode = None):
        grade_value = re.sub(RaceGrade.REGEX, "", str(grade or "").title())

        if grade_value.isdigit():
            if not 1 <= int(grade_value) < 4:
                raise ValueError(f"Grade must be between 1 and 3, not {grade}")
        elif grade_value and grade_value != "Listed":
            raise ValueError(f"Grade must be a number or 'Listed', not {grade}")

        code_from_grade = {
            "grade": RacingCode.NATIONAL_HUNT,
            "group": RacingCode.FLAT,
            "default": None,
        }[next((x for x in ["grade", "group"] if x in str(grade).lower()), "default")]

        if code_from_grade and racing_code and code_from_grade != racing_code:
            raise ValueError(
                f"{grade} conflicts with value for racing code: {racing_code.value}"
            )

        self.value = int(grade_value) if grade_value.isdigit() else grade_value
        self.racing_code = code_from_grade or racing_code or RacingCode.FLAT

    def __repr__(self):
        return f"<RaceGrade: {self.value}>"

    def __str__(self):
        if not self.value:
            return ""

        title = "Grade" if self.racing_code == RacingCode.NATIONAL_HUNT else "Group"
        return "Listed" if not str(self.value).isdigit() else f"{title} {self.value}"

    def __bool__(self):
        return bool(self.value)

    def __eq__(self, other):
        if isinstance(other, RaceGrade):
            return self.value == other.value

        if isinstance(other, int):
            return self.value != "Listed" and self.value == other

        return False

    def __lt__(self, other):
        if not self.value:
            return other.value

        if not str(self.value).isdigit():
            return str(other.value).isdigit()

        return str(other.value).isdigit() and self.value > other.value

    def __gt__(self, other):
        if not self.value:
            return False

        return str(self.value).isdigit() and (
            not other.value
            or not str(other.value).isdigit()
            or self.value < other.value
        )
